//--------------------------------------------------------------------------------------------------------------------//
//                                                FILE ACCESS INTERFACE                                               //
// 09/02/02, Mythos                                                                                                   //
//--------------------------------------------------------------------------------------------------------------------//

#ifndef   _MYTHOS_FILE_H_
#define   _MYTHOS_FILE_H_
#pragma    once

//----------------------------------------------- INCLUDES -----------------------------------------------------------//

#include  "Global/BasicTypes.h"

//----------------------------------------------- CLASSES ------------------------------------------------------------//

namespace Mythos
{
  class CAllocator;
}

//--------------------------------------------------------------------------------------------------------------------//
//                                                IFile                                                               //
//--------------------------------------------------------------------------------------------------------------------//

namespace Mythos
{
  class IFile
  {
  // public enums

  public:

    typedef enum
      {
      _READ_                  = 1 << 0,
      _WRITE_                 = 1 << 1,
      _APPEND_                = 1 << 2,
      _BINARY_                = 1 << 3,
      _AUTOSAVE_              = 1 << 4,                     // MemFile: Close calls Save when writing

      _READ_TEXT_             = _READ_,
      _READ_BINARY_           = _READ_   | _BINARY_,
      _WRITE_TEXT_            = _WRITE_,
      _WRITE_BINARY_          = _WRITE_  | _BINARY_,
      _READ_WRITE_TEXT_       = _READ_   | _WRITE_,
      _READ_WRITE_BINARY_     = _READ_   | _WRITE_ | _BINARY_,
      _APPEND_TEXT_           = _APPEND_,
      _APPEND_BINARY_         = _APPEND_ | _BINARY_,
      } FILE_MODE;

  // public methods

  public:

    // constructors & destructor

                    IFile               (void) {}
    virtual        ~IFile               (void) {}

    // operations

    virtual bool    Open                (const char* pszName, const FILE_MODE Mode,CAllocator* pAlloc=NULL) = 0;
    virtual u32     Read                (      void* pvBuffer,const u32 u32Size) = 0;
    virtual u32     Write               (const void* pvBuffer,const u32 u32Size) = 0;
    virtual bool    Close               (void) = 0;

    virtual u32     Seek                (const u32 u32Pos,const u32 u32Orig) = 0;
    virtual u32     Tell                (void) = 0;
    virtual u32     GetSize             (void) = 0;
    virtual bool    Eof                 (void) const = 0;
    virtual bool    Error               (void) const = 0;
    virtual bool    Flush               (void) = 0;

    virtual u32     GetChar             (void) = 0;
    virtual bool    GetString           (char* pszString,const u32 u32Len) = 0;
    virtual bool    PutChar             (const u32 u32Char)     = 0;
    virtual bool    PutString           (const char* pszString) = 0;

    virtual void*   Load                (const char* pszName,u32* pu32Size=NULL,CAllocator* pAlloc=NULL) = 0;
    virtual bool    Save                (const char* pszName,const void* pvBuffer,const u32 u32Size)     = 0;
  };
}

//--------------------------------------------------------------------------------------------------------------------//

#endif // _MYTHOS_FILE_H_
