//--------------------------------------------------------------------------------------------------------------------//
//                                                FILE ACCESS WITH C LIB                                              //
// 08/02/02, Mythos                                                                                                   //
//--------------------------------------------------------------------------------------------------------------------//

//----------------------------------------------- INCLUDES -----------------------------------------------------------//

#include  "stdafx.h"
#include  "FileC.h"
#ifdef    _DEBUG
#include  "FileC.inl"
#endif

#include  "Memory/Allocators/Allocator.h"
#include  "Global/Typedefs.h"

//--------------------------------------------------------------------------------------------------------------------//
//                                                OPERATIONS                                                          //
//--------------------------------------------------------------------------------------------------------------------//

//----------------------------------------------- Open ---------------------------------------------------------------//
// open given file
// 09/02/02, Mythos
// in : name,mode,allocator
// out: OK?
//--------------------------------------------------------------------------------------------------------------------//

bool Mythos::CFileC::Open(const char* pszName,const FILE_MODE Mode,CAllocator*)
  {
  if(!pszName) return false;
  if(m_pFile) Close();                                      // previous file

  char szMode[8] = { 0 };
  if(Mode & _READ_)
    {
    String::StrCat(szMode,"r");
    if(Mode & _WRITE_) String::StrCat(szMode,"+");
    }
  else if(Mode & _WRITE_) String::StrCat(szMode,"w");

  if(Mode & _APPEND_) String::StrCat(szMode,"a");
  if(Mode & _BINARY_) String::StrCat(szMode,"b");
  else                String::StrCat(szMode,"t");

  m_pFile = fopen(pszName,szMode);
  return(m_pFile != NULL);
  }

//----------------------------------------------- GetSize ------------------------------------------------------------//
// return file size
// 09/02/02, Mythos
// in :
// out: size in bytes
//--------------------------------------------------------------------------------------------------------------------//

u32 Mythos::CFileC::GetSize()
  {
  if(!m_pFile) return 0;
  DWORD dwPos  = Tell();                                    // save current position
  if(dwPos  == -1) return 0;                                // Tell failed

  if(Seek(0,SEEK_END)) return 0;
  DWORD dwSize = Tell();
  Seek(dwPos,SEEK_SET);                                     // restore position

  if(dwSize == -1) return 0;                                // Tell failed
  return dwSize;
  }

//----------------------------------------------- Load ---------------------------------------------------------------//
// load given file
// 09/02/02, Mythos
// in : name,returned size,allocator
// out: memory address
//--------------------------------------------------------------------------------------------------------------------//

void* Mythos::CFileC::Load(const char* pszName,u32* pu32Size,CAllocator* pAlloc)
  {
  if(!pszName) return NULL;
  if(!Open(pszName,_READ_BINARY_)) return NULL;

  u32 u32Size = GetSize();
  if(!u32Size) return NULL;

  void* pvFile = Memory::Malloc(u32Size,pAlloc);            // get some memory
  if(!pvFile) return NULL;

  u32 u32Read = Read(pvFile,u32Size);                       // read file
  if(u32Read < u32Size)
    {
    Memory::Free(pvFile,pAlloc);
    return NULL;
    }

  Close();
  if(pu32Size) *pu32Size = u32Size;
  return pvFile;
  }

//----------------------------------------------- Save ---------------------------------------------------------------//
// save given block of memory
// 09/02/02, Mythos
// in : name,memory buffer,buffer size
// out: OK?
//--------------------------------------------------------------------------------------------------------------------//

bool Mythos::CFileC::Save(const char* pszName,const void* pvBuffer,const u32 u32Size)
  {
  if(!pszName || !pvBuffer) return false;
  if(!Open(pszName,_WRITE_BINARY_)) return false;

  u32 u32Written = Write(pvBuffer,u32Size);
  if(u32Written < u32Size) return false;

  Close();
  return true;
  }
