//--------------------------------------------------------------------------------------------------------------------//
//                                                FILE ACCESS WITH C LIB                                              //
// 08/02/02, Mythos                                                                                                   //
//--------------------------------------------------------------------------------------------------------------------//

#ifndef   _MYTHOS_FILEC_INL_
#define   _MYTHOS_FILEC_INL_
#pragma    once

//--------------------------------------------------------------------------------------------------------------------//
//                                                CONSTRUCTORS / DESTRUCTOR                                           //
//--------------------------------------------------------------------------------------------------------------------//

INLINE Mythos::CFileC::CFileC() : m_pFile(NULL)
  {}

//

INLINE Mythos::CFileC::~CFileC()
  {
  if(m_pFile) Close();
  }

//--------------------------------------------------------------------------------------------------------------------//
//                                                OPERATIONS                                                          //
//--------------------------------------------------------------------------------------------------------------------//

INLINE u32 Mythos::CFileC::Read(void* pvBuffer,const u32 u32Size)
  {
  if(!m_pFile) return 0;
  return fread(pvBuffer,1,u32Size,m_pFile);
  }

//

INLINE u32 Mythos::CFileC::Write(const void* pvBuffer,const u32 u32Size)
  {
  if(!m_pFile) return 0;
  return fwrite(pvBuffer,1,u32Size,m_pFile);
  }

//

INLINE bool Mythos::CFileC::Close()
  {
  if(!m_pFile) return true;                                 // already closed
  fclose(m_pFile);
  m_pFile = NULL;
  return true;
  }

//

INLINE u32 Mythos::CFileC::Seek(const u32 u32Pos,const u32 u32Orig)
  {
  if(!m_pFile) return 1;                                    // !=0 : error
  return fseek(m_pFile,u32Pos,u32Orig);
  }

//

INLINE u32 Mythos::CFileC::Tell()
  {
  if(!m_pFile) return -1;                                   // -1 : error
  return ftell(m_pFile);
  }

//

INLINE bool Mythos::CFileC::Eof() const
  {
  if(!m_pFile) return true;                                 // no file => EOF
  return(feof(m_pFile) != 0);
  }

//

INLINE bool Mythos::CFileC::Error() const
  {
  if(!m_pFile) return true;                                 // no file => error
  return(ferror(m_pFile) != 0);
  }

//

INLINE bool Mythos::CFileC::Flush()
  {
  if(!m_pFile) return false;
  return(fflush(m_pFile) != EOF);
  }

//

INLINE u32 Mythos::CFileC::GetChar()
  {
  if(!m_pFile) return EOF;
  return fgetc(m_pFile);
  }

//

INLINE bool Mythos::CFileC::GetString(char* pszString,const u32 u32Len)
  {
  if(!m_pFile) return false;
  return(fgets(pszString,u32Len,m_pFile) != NULL);
  }

//

INLINE bool Mythos::CFileC::PutChar(const u32 u32Char)
  {
  if(!m_pFile) return false;
  return(fputc(u32Char,m_pFile) != EOF);
  }

//

INLINE bool Mythos::CFileC::PutString(const char* pszString)
  {
  if(!m_pFile) return false;
  return(fputs(pszString,m_pFile) != EOF);
  }

//--------------------------------------------------------------------------------------------------------------------//

#endif // _MYTHOS_FILEC_INL_
