//--------------------------------------------------------------------------------------------------------------------//
//                                                MEMORY FILE ACCESS WITH C LIB                                       //
// 13/02/02, Mythos                                                                                                   //
//--------------------------------------------------------------------------------------------------------------------//

#ifndef   _MYTHOS_MEMFILEC_H_
#define   _MYTHOS_MEMFILEC_H_
#pragma    once

//----------------------------------------------- INCLUDES -----------------------------------------------------------//

#include  "File.h"
#include  "Global/Defines.h"
#include  "Containers/StdString.h"

//----------------------------------------------- CLASSES ------------------------------------------------------------//

//--------------------------------------------------------------------------------------------------------------------//
//                                                CMemFileC                                                           //
//--------------------------------------------------------------------------------------------------------------------//

namespace Mythos
{
  class CMemFileC : public IFile
  {
  // public methods

  public:

    // constructors & destructor

                    CMemFileC           (void);
    virtual        ~CMemFileC           (void);

    // operations

    virtual bool    Open                (const char* pszName, const FILE_MODE Mode,CAllocator* pAlloc=NULL);
    virtual u32     Read                (      void* pvBuffer,const u32 u32Size);
    virtual u32     Write               (const void* pvBuffer,const u32 u32Size);
    virtual bool    Close               (void);

    virtual u32     Seek                (const u32 u32Pos,const u32 u32Orig);
    virtual u32     Tell                (void);
    virtual u32     GetSize             (void);
    virtual bool    Eof                 (void) const;
    virtual bool    Error               (void) const;
    virtual bool    Flush               (void);

    virtual u32     GetChar             (void);
    virtual bool    GetString           (char* pszString,const u32 u32Len);
    virtual bool    PutChar             (const u32 u32Char);
    virtual bool    PutString           (const char* pszString);

    virtual void*   Load                (const char* pszName,u32* pu32Size=NULL,CAllocator* pAlloc=NULL);
    virtual bool    Save                (const char* pszName,const void* pvBuffer,const u32 u32Size);

  // protected methods

  protected:

    virtual void    Init                (void);
    virtual void    DeleteBuffer        (void);
    virtual bool    Expand              (const u32 u32NewSize);

  // protected members

  protected:

    CAllocator*     m_pAlloc;
    u8*             m_pu8File;
    u32             m_u32Size;                              // buffer size (= mem allocated)
    u32             m_u32Used;                              // bytes  used (= file size)
    u8*             m_pu8CurrPos;

    bool            m_boClosed;
    bool            m_boEOF;
    bool            m_boError;
    u32             m_u32Flags;

    CStdString      m_strName;
  };
}

//----------------------------------------------- INLINES ------------------------------------------------------------//

#ifndef   _DEBUG
#include  "MemFileC.inl"
#endif

//--------------------------------------------------------------------------------------------------------------------//

#endif // _MYTHOS_MEMFILEC_H_
